#~*~ coding: utf-8 ~*~
from django.contrib import admin
from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from virtenviro.shop.models import Product,\
    Category, Property, PropertyType,\
    Image, ImageType, Manufacturer, PropertySlug, PropertyTypeCategoryRelation,\
    ImageTypeCategoryRelation, Currency, File, Seller,\
    OrderStatus, Order, ProductOrderRelation


class APInline(admin.TabularInline):
    model = Property
    extra = 4

    def formfield_for_foreignkey(self, db_field, request = None, **kwargs):
        field = super(APInline, self).formfield_for_foreignkey(db_field, request, **kwargs)
        category_id = None
        if request.GET.has_key('category'):
            category_id = request.GET['category']
        else:
            mod = False
            for val in request.path.split('/'):
                try:
                    id = int(val)
                    category_id = Product.objects.get(id = id).category.id
                    mod = True
                    break
                except:
                    pass
        if category_id:
            if db_field.name == 'additional_property_type':
                field.queryset = field.queryset.filter(category=category_id)
        return field
    
class APTInline(admin.TabularInline):
    model = PropertyType.category.through
    extra = 5
    
class ITInline(admin.TabularInline):
    model = ImageType.category.through
    extra = 5


class ImageInline(admin.TabularInline):
    model = Image
    extra = 5

    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        field = super(ImageInline, self).formfield_for_foreignkey(db_field, request, **kwargs)
        category_id = None
        if request.GET.has_key('category'):
            category_id = request.GET['category']
        else:
            mod = False
            for val in request.path.split('/'):
                try:
                    id = int(val)
                    category_id = Product.objects.get(id = id).category.id
                    mod = True
                    break
                except:
                    pass
        if category_id and db_field.name == 'image_type':
            field.queryset = field.queryset.filter(category = category_id)
        
        return field


class ProductAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(ProductAdminForm, self).__init__(*args, **kwargs)
        self.fields['description'].widget = forms.Textarea(attrs={'class': 'ckeditor'})

    class Meta:
        model = Product
        exclude = ()


class ProductAdmin(admin.ModelAdmin):
    list_display = ('name', 'price', 'ordering', 'view_count')
    list_filter = ('category', )
    list_editable = ('price', 'ordering')
    search_fields = ['name', 'category__name', 'slug']
    inlines = [
        APInline,
        ImageInline,
    ]

    form = ProductAdminForm
    valid_lookups = ('parent')


    def get_form(self, request, obj=None, **kwargs):
        form = super(ProductAdmin, self).get_form(request, obj, **kwargs)
        return form

    def lookup_allowed(self, lookup, value):
        if lookup.startswith(self.valid_lookups):
            return True
        
        return super(ProductAdmin, self).lookup_allowed(lookup, value)
        
    class Media:
        try:
            if settings.CKEDITOR:
                js = (
                    '/static/ckeditor/ckeditor.js',
                    '/static/filebrowser/js/FB_CKEditor.js',
                    '/static/js/ckeditor.js',
                )
                css = {'all': ('/static/css/ckeditor.css',), }
        except AttributeError:
            pass

    
class CategoryAdmin(admin.ModelAdmin):
    search_fields = ['name']
    list_display = ('name', 'production_link', 'add_product_link')
    inlines = [APTInline, ITInline]
    
    def add_product_link(self, obj):
        return ('<a href="/admin/shop/product/add/?category=%s" class="addlink">%s</a>' % (obj.id, u'Добавить продукт'))
    add_product_link.allow_tags = True
    
    def production_link(self, obj):
        return ('<a href="/admin/shop/product/?category__id__exact=%s&parent__isnull=True">%s</a>' % (obj.id, u'Продукция'))
    production_link.allow_tags = True


class PropertySlugInline(admin.StackedInline):
    model = PropertySlug
    extra = 0

'''
    if not value_choices is None:
        formfield_overrides = {
            'value': {
                'widget': forms.Select(choices=value_choices)
            }
        }

    def set_value(self, args, kwargs):
        if not self._parent_instance is None:
            self.value_choices = Property.objects.grouped(property_type=self._parent_instance)
'''
#todo: create action to generate slugs for selected property_types properties


class PropertyTypeAdmin(admin.ModelAdmin):
    search_fields = ['name']
    inlines = [
        PropertySlugInline,
    ]


class PropertyTypeCategoryRelationAdmin(admin.ModelAdmin):
    list_display = ('property_type', 'category', 'slug', 'max_count')
    list_editable = ('slug', 'max_count')


class SellerAdmin(admin.ModelAdmin):
    list_display = ('name', 'ordering')
    list_editable = ('ordering',)


class ProductOrderRelationInline(admin.TabularInline):
    model = ProductOrderRelation
    extra = 0


class OrderAdmin(admin.ModelAdmin):
    list_display = ('id', 'user_field', 'created_time', 'modified_time', 'status', 'summary_count', 'summary_products')
    list_filter = ('status', 'created_time', 'modified_time',)

    def summary_count(self, obj):
        return obj.summary_count()
    summary_count.short_description = _('All products count')

    def summary_products(self, obj):
        return obj.summary_products()
    summary_products.short_description = _('Products count')

    def user_field(self, obj):
        return obj.user.email
    user_field.short_description = _('User')

    inlines = [
        ProductOrderRelationInline]

admin.site.register(Product, ProductAdmin)
admin.site.register(Category, CategoryAdmin)
admin.site.register(Property)
admin.site.register(PropertyType, PropertyTypeAdmin)
admin.site.register(Image)
admin.site.register(ImageType)
admin.site.register(Manufacturer)
admin.site.register(PropertySlug)
admin.site.register(PropertyTypeCategoryRelation, PropertyTypeCategoryRelationAdmin)
admin.site.register(ImageTypeCategoryRelation)
admin.site.register(Currency)
admin.site.register(File)
admin.site.register(Seller, SellerAdmin)
admin.site.register(Order, OrderAdmin)
admin.site.register(OrderStatus)
admin.site.register(ProductOrderRelation)


# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-26 05:12
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('slug', models.CharField(blank=True, max_length=60, verbose_name='Slug')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('meta_title', models.CharField(blank=True, max_length=255, null=True, verbose_name='Meta title')),
                ('meta_keywords', models.CharField(blank=True, max_length=255, null=True, verbose_name='Meta keywords')),
                ('meta_description', models.CharField(blank=True, max_length=255, null=True, verbose_name='Meta description')),
            ],
            options={
                'ordering': ['title'],
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
            },
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('slug', models.CharField(blank=True, max_length=60, null=True, unique=True, verbose_name='Slug')),
                ('intro', models.TextField(blank=True, null=True, verbose_name='Intro')),
                ('content', models.TextField(verbose_name='Content')),
                ('origin', models.URLField(blank=True, null=True, verbose_name="Origin's url address")),
                ('published', models.BooleanField(default=False, verbose_name='Published')),
                ('pub_datetime', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Created datetime')),
                ('last_modified', models.DateTimeField(auto_now=True, verbose_name='Last modified datetime')),
                ('meta_title', models.CharField(blank=True, max_length=255, null=True, verbose_name='Meta title')),
                ('meta_keywords', models.CharField(blank=True, max_length=255, null=True, verbose_name='Meta keywords')),
                ('meta_description', models.CharField(blank=True, max_length=255, null=True, verbose_name='Meta description')),
                ('author', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Author')),
                ('category', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='news.Category', verbose_name='Category')),
            ],
            options={
                'ordering': ('-pub_datetime', 'title'),
                'verbose_name': 'News post',
                'verbose_name_plural': 'News posts',
            },
        ),
    ]



#~*~ coding: utf-8 ~*~
from django.shortcuts import render, redirect
from django.contrib.auth.decorators import login_required
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.models import User
from virtenviro.registration.models import UserProfile
from virtenviro.registration.forms import UserForm, UserProfileForm


def signup(request):
    """Регистрация"""
    user = User()
    user_profile = UserProfile()
    if request.method == "POST":
        user_form = UserForm(request.POST, instance=user)
        user_profile_form = UserProfileForm(request.POST, instance=user_profile)
        if user_form.is_valid() and user_profile_form.is_valid():
            user_data = user_form.cleaned_data
            user.username = user_data['email']
            user.first_name = user_data['firstname']
            user.last_name = user_data['lastname']
            user.email = user_data['email']
            user.set_password(user_data['pass1'])
            user.save()

            user_profile = user.get_profile()
            user_profile_data = user_profile_form.cleaned_data
            user_profile.weight = user_profile_data['weight']
            user_profile.save()
            user = authenticate(username=user_data['username'], password=user_data['pass1'] )
            login(request, user)
            return redirect("/accounts/profile/")
    else:
        user_form = UserForm( instance = user )
        user_profile_form = UserProfileForm( instance = user_profile )
    return render(request,
                  "accounts/signup.html",
                  {
                      "user_": user,
                      "user_profile": user_profile,
                      "user_form": user_form,
                      "user_profile_form": user_profile_form})

@login_required
def profile(request):
    """Профиль текущего пользователя"""
    user = request.user
    return render(request,
                  "accounts/card.html",
                  {
                      "user": user})


@login_required
def user_card(request):
    """Профиль текущего пользователя"""
    user = request.user
    return render(request,
                  "accounts/card.html",
                  {"user": user})


@login_required
def logout(request):
    goto = request.GET.get("goto", "/")
    return redirect(goto)


# ~*~ coding: utf-8 ~*~
"""Pickle field implementation for Django."""

from copy import deepcopy
from base64 import b64encode, b64decode
from zlib import compress, decompress
try:
    from cPickle import loads, dumps
except ImportError:
    from pickle import loads, dumps

from django.db import models
from django.utils.encoding import force_unicode

from virtenviro.picklefield import DEFAULT_PROTOCOL


class PickledObject(str):
    """
    A subclass of string so it can be told whether a string is a pickled
    object or not (if the object is an instance of this class then it must
    [well, should] be a pickled one).
    Only really useful for passing pre-encoded values to ``default``
    with ``dbsafe_encode``, not that doing so is necessary. If you
    remove PickledObject and its references, you won't be able to pass
    in pre-encoded values anymore, but you can always just pass in the
    python objects themselves.
    """


def dbsafe_encode(value, compress_object=False, pickle_protocol=DEFAULT_PROTOCOL):
    # We use deepcopy() here to avoid a problem with cPickle, where dumps
    # can generate different character streams for same lookup value if
    # they are referenced differently.
    # The reason this is important is because we do all of our lookups as
    # simple string matches, thus the character streams must be the same
    # for the lookups to work properly. See tests.py for more information.
    if not compress_object:
        value = b64encode(dumps(deepcopy(value), pickle_protocol))
    else:
        value = b64encode(compress(dumps(deepcopy(value), pickle_protocol)))
    return PickledObject(value)


def dbsafe_decode(value, compress_object=False):
    if not compress_object:
        value = loads(b64decode(value))
    else:
        value = loads(decompress(b64decode(value)))
    return value


class PickledObjectField(models.Field):
    """
    A field that will accept *any* python object and store it in the
    database. PickledObjectField will optionally compress its values if
    declared with the keyword argument ``compress=True``.
    Does not actually encode and compress ``None`` objects (although you
    can still do lookups using None). This way, it is still possible to
    use the ``isnull`` lookup type correctly.
    """

    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        self.compress = kwargs.pop('compress', False)
        self.protocol = kwargs.pop('protocol', DEFAULT_PROTOCOL)
        kwargs.setdefault('editable', False)
        super(PickledObjectField, self).__init__(*args, **kwargs)

    def get_default(self):
        """
        Returns the default value for this field.
        The default implementation on models.Field calls force_unicode
        on the default, which means you can't set arbitrary Python
        objects as the default. To fix this, we just return the value
        without calling force_unicode on it. Note that if you set a
        callable as a default, the field will still call it. It will
        *not* try to pickle and encode it.
        """
        if self.has_default():
            if callable(self.default):
                return self.default()
            return self.default
        # If the field doesn't have a default, then we punt to models.Field.
        return super(PickledObjectField, self).get_default()

    def to_python(self, value):
        """
        B64decode and unpickle the object, optionally decompressing it.
        If an error is raised in de-pickling and we're sure the value is
        a definite pickle, the error is allowed to propogate. If we
        aren't sure if the value is a pickle or not, then we catch the
        error and return the original value instead.
        """
        if value is not None:
            try:
                value = dbsafe_decode(value, self.compress)
            except:
                # If the value is a definite pickle; and an error is raised in
                # de-pickling it should be allowed to propogate.
                if isinstance(value, PickledObject):
                    raise
        return value

    def get_db_prep_value(self, value):
        """
        Pickle and b64encode the object, optionally compressing it.
        The pickling protocol is specified explicitly (by default 2),
        rather than as -1 or HIGHEST_PROTOCOL, because we don't want the
        protocol to change over time. If it did, ``exact`` and ``in``
        lookups would likely fail, since pickle would now be generating
        a different string.
        """
        if value is not None and not isinstance(value, PickledObject):
            # We call force_unicode here explicitly, so that the encoded string
            # isn't rejected by the postgresql_psycopg2 backend. Alternatively,
            # we could have just registered PickledObject with the psycopg
            # marshaller (telling it to store it like it would a string), but
            # since both of these methods result in the same value being stored,
            # doing things this way is much easier.
            value = force_unicode(dbsafe_encode(value, self.compress, self.protocol))
        return value

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_db_prep_value(value)

    def get_internal_type(self):
        return 'TextField'

    def get_db_prep_lookup(self, lookup_type, value, connection=None, prepared=False):
        if lookup_type not in ['exact', 'in', 'isnull']:
            raise TypeError('Lookup type %s is not supported.' % lookup_type)
        # The Field model already calls get_db_prep_value before doing the
        # actual lookup, so all we need to do is limit the lookup types.
        try:
            return super(PickledObjectField, self).get_db_prep_lookup(
                lookup_type, value, connection=connection, prepared=prepared)
        except TypeError:
            # Try not to break on older versions of Django, where the
            # `connection` and `prepared` parameters are not available.
            return super(PickledObjectField, self).get_db_prep_lookup(
                lookup_type, value)


# South support; see http://south.aeracode.org/docs/tutorial/part4.html#simple-inheritance
try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    pass
else:
    add_introspection_rules([], [r"^picklefield\.fields\.PickledObjectField"])



# ~*~ coding: utf-8 ~*~
__author__ = 'Kamo Petrosyan'
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.utils import timezone
from django.conf import settings

LANGUAGE_CODE = getattr(settings, 'LANGUAGE_CODE', 'ru')
LANGUAGES = getattr(settings, 'LANGUAGES', ('ru',))


class AbstractSeo(models.Model):
    # META FIELDS
    meta_title = models.CharField(max_length=250, verbose_name=_('Meta Title'), null=True, blank=True)
    meta_keywords = models.TextField(verbose_name=_('Meta Keywords'), null=True, blank=True)
    meta_description = models.TextField(verbose_name=_('Meta Description'), null=True, blank=True)

    class Meta:
        abstract = True


class AbstractContent(models.Model):
    title = models.CharField(max_length=250, verbose_name=_('Title'))
    h1 = models.CharField(max_length=250, verbose_name=_('H1 tag'), null=True, blank=True)
    intro = models.TextField(verbose_name=_('Intro'), null=True, blank=True)
    content = models.TextField(verbose_name=_('Content'), null=True, blank=True)
    template = models.ForeignKey('content.Template', verbose_name=_('Template'), null=True, blank=True)

    def get_template(self):
        if not self.template is None:
            return self.template
        else:
            return self.parent.template

    class Meta:
        abstract = True


class AbstractContentMultilingual(models.Model):
    language = models.CharField(max_length=10, verbose_name=_('Language'), choices=LANGUAGES, default=LANGUAGE_CODE)

    class Meta:
        abstract = True


from django.contrib import admin
from django import forms
from django.conf import settings

from virtenviro.news.models import Post, Category


class PostAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(PostAdminForm, self).__init__(*args, **kwargs)
        self.fields['content'].widget.attrs['class'] = 'ckeditor'
        self.fields['intro'].widget = forms.Textarea(attrs={'class': 'ckeditor'})

    class Meta:
        model = Post
        fields = [
            'title',
            'category',
            'slug',
            'intro',
            'content',
            'origin',
            'author',
            'published',
            'pub_datetime',
            'meta_title',
            'meta_keywords',
            'meta_description',
        ]


class PostAdmin(admin.ModelAdmin):
    search_fields = ['title', 'intro', 'content']
    list_filter = ('pub_datetime', )
    form = PostAdminForm

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == 'author':
            kwargs['initial'] = request.user.id

        if db_field.name == 'last_modified_by':
            kwargs['initial'] = request.user.id
        return super(PostAdmin, self).formfield_for_foreignkey(
            db_field, request, **kwargs
        )

    class Media:
        try:
            if settings.CKEDITOR:
                css = {'all': ('/static/css/ckeditor.css',), }
                js = (
                    '/static/ckeditor/ckeditor.js',
                    '/static/filebrowser/js/FB_CKEditor.js',
                    '/static/js/ckeditor.js',
                )
        except AttributeError:
            pass

admin.site.register(Post, PostAdmin)
admin.site.register(Category)


#~*~ coding: utf-8 ~*~
from django.db import models
from django.utils.translation import ugettext_lazy as _

ENCTYPES = (
    ('application/x-www-form-urlencoded', 'application/x-www-form-urlencoded'),
    ('multipart/form-data', 'multipart/form-data'),
    ('text/plain', 'text/plain')
)
METHODS = (
    ('GET', 'GET'),
    ('POST', 'POST')
)
TARGETS = (
    ('_blank', '_blank'),
    ('_self', '_self'),
    ('_parent', '_parent'),
    ('_top', '_top')
)


class Form(models.Model):
    title = models.CharField(max_length=255, verbose_name=_('Title'))
    name = models.CharField(max_length=255, verbose_name=_('Name'))
    action = models.CharField(max_length=5000, verbose_name=_('Action'), null=True, blank=True)
    enctype = models.CharField(max_length=40, verbose_name=_('Enctype'), default='application/x-www-form-urlencoded',
                               choices=ENCTYPES)
    method = models.CharField(max_length=4, verbose_name=_('Method'),
                              help_text=_('Data transfer method'),
                              choices=METHODS, default='GET')
    novalidate = models.BooleanField(default=False, verbose_name=_('Do not validate'))
    target = models.CharField(max_length=7, verbose_name=_('Target'), choices=TARGETS, null=True, blank=True)

    def __unicode__(self):
        return self.title

    class Meta:
        verbose_name = _('Form')
        verbose_name_plural = _('Forms')


class Field(models.Model):
    TAGS = (
        ('input', 'input'),
        ('textarea', 'textarea'),
        ('button', 'button'),
    )
    ALIGNS = (
        ('bottom', _('bottom')),
        ('left', _('left')),
        ('middle', _('middle')),
        ('right', _('right')),
        ('top', _('top')),
    )
    TYPES = (
        ('text', 'text'),
        ('button', 'button'),
        ('checkbox', 'checkbox'),
        ('file', 'file'),
        ('hidden', 'hidden'),
        ('hidden', 'hidden'),
        ('image', 'image'),
        ('password', 'password'),
        ('radio', 'radio'),
        ('reset', 'reset'),
        ('submit', 'submit'),
        ('textarea', 'textarea'),
        ('email', 'email'),
        ('number', 'number'),
        ('range', 'range'),
        ('search', 'search'),
        ('tel', 'tel'),
        ('url', 'url'),
    )
    title = models.CharField(max_length=255, verbose_name=_('Title'))
    name = models.CharField(max_length=255, verbose_name=_('Name'))
    form = models.ForeignKey(Form, verbose_name=_('Form'))
    value = models.TextField(verbose_name=_('Default value'), )

    in_form = models.BooleanField(default=True, verbose_name=_('Is input in form?'))
    field_tag = models.CharField(max_length=10, verbose_name=_('Field tag'), default='input', choices=TAGS)
    field_type = models.CharField(max_length=200, verbose_name=_('Type'), choices=TYPES, default='text')
    align = models.CharField(max_length=6, verbose_name=_('Align'),
                             help_text=_('Image input alignment'), choices=ALIGNS, null=True, blank=True)
    alt = models.CharField(max_length=255, verbose_name=_('Image alt text'))
    autofocus = models.BooleanField(verbose_name=_('Auto focus'), default=False)
    border = models.IntegerField(verbose_name=_('Image input border'), null=True, blank=True)
    chacked = models.BooleanField(verbose_name=_('Checked'), default=False)
    disabled = models.BooleanField(verbose_name=_('Disabled'), default=False)

    formaction = models.CharField(max_length=5000, verbose_name=_('Action'), null=True, blank=True)
    formenctype = models.CharField(max_length=40, verbose_name=_('Enctype'), default='application/x-www-form-urlencoded',
                               choices=ENCTYPES, null=True, blank=True)
    formmethod = models.CharField(max_length=4, verbose_name=_('Method'),
                              help_text=_('Data transfer method'),
                              choices=METHODS, default='GET', null=True, blank=True)
    formnovalidate = models.BooleanField(default=False, verbose_name=_('Do not validate'))
    formtarget = models.CharField(max_length=7, verbose_name=_('Target'), choices=TARGETS, null=True, blank=True)

    datalist = models.TextField(verbose_name=_('List'), null=True, blank=True)
    maxlength = models.IntegerField(verbose_name=_('Max length'), null=True, blank=True)
    max_val = models.IntegerField(verbose_name=_('Max value'), default=0)
    min_val = models.IntegerField(verbose_name=_('Min value'), default=0)
    step = models.IntegerField(verbose_name=_('Step'), default=0)

    multiple = models.BooleanField(default=False, verbose_name=_('Multiple file upload'))
    pattern = models.CharField(max_length=255, verbose_name=_('Pattern'), null=True, blank=True)
    placeholder = models.CharField(max_length=255, verbose_name=_('Placeholder'), null=True, blank=True)
    required = models.BooleanField(default=False, verbose_name=_('Required'))

    styles = models.CharField(max_length=255, verbose_name=_('Styles'), null=True, blank=True)
    ordering = models.IntegerField(default=999, verbose_name=_('Ordering'))

    def clean(self):
        if not self.field_type == 'image':
            self.align = None
            self.alt = None
        if not self.field_type == 'checkbox':
            self.chacked = False

        if not self.min_val and self.max_val < self.min_val:
            self.max_val = self.min_val

    '''
    accept Устанавливает фильтр на типы файлов, которые вы можете отправить через поле загрузки файлов.
    align Определяет выравнивание изображения.
    alt Альтернативный текст для кнопки с изображением.
    autofocus Устанавливает фокус в поле формы.
    border Толщина рамки вокруг изображения.
    checked Предварительно активированный переключатель или флажок.
    disabled Блокирует доступ и изменение элемента.

    form Связывает поле с формой по её идентификатору.
    formaction Определяет адрес обработчика формы.
    formenctype Устанавливает способ кодирования данных формы при их отправке на сервер.
    formmethod Сообщает браузеру каким методом следует передавать данные формы на сервер.
    formnovalidate Отменяет встроенную проверку данных на корректность.
    formtarget Определяет окно или фрейм в которое будет загружаться результат, возвращаемый обработчиком формы.

    list Указывает на список вариантов, которые можно выбирать при вводе текста.
    max Верхнее значение для ввода числа или даты.
    maxlength Максимальное количество символов разрешенных в тексте.
    min Нижнее значение для ввода числа или даты.

    multiple Позволяет загрузить несколько файлов одновременно.
    name Имя поля, предназначено для того, чтобы обработчик формы мог его идентифицировать.
    pattern Устанавливает шаблон ввода.
    placeholder Выводит подсказывающий текст.
    readonly Устанавливает, что поле не может изменяться пользователем.
    required Обязательное для заполнения поле.
    size Ширина текстового поля.
    src Адрес графического файла для поля с изображением.
    step Шаг приращения для числовых полей.
    tabindex Определяет порядок перехода между элементами с помощью клавиши Tab.
    type Сообщает браузеру, к какому типу относится элемент формы.
    value Значение элемента.
    '''

    def __unicode__(self):
        return self.title


class Action(models.Model):
    name = models.CharField(max_length=255, verbose_name=_('Name'))

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Action')
        verbose_name_plural = _('Actions')
